@@ -117,6 +117,16 @@
 #'   odds ratios and log risk ratios will be shown.
 #' @param pscale A numeric defining a scaling factor for printing of
 #'   risk differences.
+#' @param text.fixed A character string used in printouts and forest
+#'   plot to label the pooled fixed effect estimate.
+#' @param text.random A character string used in printouts and forest
+#'   plot to label the pooled random effects estimate.
+#' @param text.predict A character string used in printouts and forest
+#'   plot to label the prediction interval.
+#' @param text.w.fixed A character string used to label weights of
+#'   fixed effect model.
+#' @param text.w.random A character string used to label weights of
+#'   random effects model.
 #' @param title Title of meta-analysis / systematic review.
 #' @param complab Comparison label.
 #' @param outclab Outcome label.
@@ -321,20 +331,32 @@
 #' In rare settings with very homogeneous treatment estimates, the
 #' Hartung-Knapp variance estimate can be arbitrarily small resulting
 #' in a very narrow confidence interval (Knapp and Hartung, 2003;
-#' Wiksten et al., 2016). In such cases, an \emph{ad hoc} variance
-#' correction has been proposed by utilising the variance estimate
-#' from the classic random effects model (Knapp and Hartung,
-#' 2003). Argument \code{adhoc.hakn} can be used to choose the
-#' \emph{ad hoc} method:
+#' Wiksten et al., 2016). In such cases, an
+#' \emph{ad hoc} variance correction has been proposed by utilising
+#' the variance estimate from the classic random effects model with
+#' the HK method (Knapp and Hartung, 2003; IQWiQ, 2020). An
+#' alternative approach is to use the wider confidence interval of
+#' classic fixed or random effects meta-analysis and the HK method
+#' (Wiksten et al., 2016; Jackson et al., 2017).
+#'
+#' Argument \code{adhoc.hakn} can be used to choose the \emph{ad hoc}
+#' method:
 #' \tabular{ll}{
-#' \bold{Argument}\tab \bold{\emph{Ad hoc} method} \cr 
+#' \bold{Argument}\tab \bold{\emph{Ad hoc} method} \cr
 #' \code{adhoc.hakn = ""}\tab not used \cr
-#' \code{adhoc.hakn = "se"}\tab used if HK standard error is smaller than
-#'  standard error \cr
-#'  \tab from classic random effects model (Knapp and Hartung, 2003) \cr
-#' \code{adhoc.hakn = "ci"}\tab used if HK confidence interval is
-#'  narrower than CI from \cr
-#'  \tab classic random effects model with DL estimator (IQWiG, 2020)
+#' \code{adhoc.hakn = "se"}\tab use variance correction if HK standard
+#'  error is smaller \cr
+#'  \tab than standard error from classic random effects
+#'  \cr
+#'  \tab meta-analysis (Knapp and Hartung, 2003) \cr
+#' \code{adhoc.hakn = "iqwig6"}\tab use variance correction if HK
+#'  confidence interval \cr
+#'  \tab is narrower than CI from classic random effects model \cr
+#'  \tab with DerSimonian-Laird estimator (IQWiG, 2020) \cr
+#' \code{adhoc.hakn = "ci"}\tab use wider confidence interval of
+#'  classic random effects \cr
+#'  \tab and HK meta-analysis \cr
+#'  \tab (Hybrid method 2 in Jackson et al., 2017)
 #' }
 #' }
 #' 
@@ -417,17 +439,17 @@
 #'   (fixed effect model).}
 #' \item{lower.fixed, upper.fixed}{Lower and upper confidence interval
 #'   limits (fixed effect model).}
-#' \item{zval.fixed, pval.fixed}{z-value and p-value for test of
+#' \item{statistic.fixed, pval.fixed}{z-value and p-value for test of
 #'   overall treatment effect (fixed effect model).}
 #' \item{TE.random, seTE.random}{Estimated overall treatment effect,
 #'   e.g., log risk ratio or risk difference, and standard error
 #'   (random effects model).}
 #' \item{lower.random, upper.random}{Lower and upper confidence
 #'   interval limits (random effects model).}
-#' \item{zval.random, pval.random}{z-value or t-value and
+#' \item{statistic.random, pval.random}{z-value or t-value and
 #'   corresponding p-value for test of overall treatment effect
-#'   (random effects model).}
-#' \item{prediction, level.predict}{As defined above.}
+#'   (random effects model).}  \item{prediction, level.predict}{As
+#'   defined above.}
 #' \item{seTE.predict}{Standard error utilised for prediction
 #'   interval.}
 #' \item{lower.predict, upper.predict}{Lower and upper limits of
@@ -478,8 +500,9 @@
 #'   \code{byvar} is not missing.}  \item{lower.fixed.w,
 #'   upper.fixed.w}{Lower and upper confidence interval limits in
 #'   subgroups (fixed effect model) - if \code{byvar} is not missing.}
-#' \item{zval.fixed.w, pval.fixed.w}{z-value and p-value for test of
-#'   treatment effect in subgroups (fixed effect model) - if
+#'
+#' \item{statistic.fixed.w, pval.fixed.w}{z-value and p-value for test
+#'   of treatment effect in subgroups (fixed effect model) - if
 #'   \code{byvar} is not missing.}  \item{TE.random.w,
 #'   seTE.random.w}{Estimated treatment effect and standard error in
 #'   subgroups (random effects model) - if \code{byvar} is not
@@ -487,79 +510,68 @@
 #' \item{lower.random.w, upper.random.w}{Lower and upper confidence
 #'   interval limits in subgroups (random effects model) - if
 #'   \code{byvar} is not missing.}
-#' \item{zval.random.w, pval.random.w}{z-value or t-value and
+#' \item{statistic.random.w, pval.random.w}{z-value or t-value and
 #'   corresponding p-value for test of treatment effect in subgroups
 #'   (random effects model) - if \code{byvar} is not missing.}
 #' \item{w.fixed.w, w.random.w}{Weight of subgroups (in fixed and
 #'   random effects model) - if \code{byvar} is not missing.}
-#' \item{df.hakn.w}{Degrees of freedom for test of treatment effect
+#'   \item{df.hakn.w}{Degrees of freedom for test of treatment effect
 #'   for Hartung-Knapp method in subgroups - if \code{byvar} is not
-#'   missing and \code{hakn = TRUE}.}
-#' \item{event.e.w}{Number of events in experimental group in
+#'   missing and \code{hakn = TRUE}.}  \item{event.e.w}{Number of
+#'   events in experimental group in subgroups - if \code{byvar} is
+#'   not missing.}  \item{n.e.w}{Number of observations in
+#'   experimental group in subgroups - if \code{byvar} is not
+#'   missing.}  \item{event.c.w}{Number of events in control group in
+#'   subgroups - if \code{byvar} is not missing.}  \item{n.c.w}{Number
+#'   of observations in control group in subgroups - if \code{byvar}
+#'   is not missing.}  \item{k.w}{Number of studies combined within
 #'   subgroups - if \code{byvar} is not missing.}
-#' \item{n.e.w}{Number of observations in experimental group in
+#'   \item{k.all.w}{Number of all studies in subgroups - if
+#'   \code{byvar} is not missing.}  \item{Q.w.fixed}{Overall within
+#'   subgroups heterogeneity statistic Q (based on fixed effect model)
+#'   - if \code{byvar} is not missing.}  \item{Q.w.random}{Overall
+#'   within subgroups heterogeneity statistic Q (based on random
+#'   effects model) - if \code{byvar} is not missing (only calculated
+#'   if argument \code{tau.common} is TRUE).}  \item{df.Q.w}{Degrees
+#'   of freedom for test of overall within subgroups heterogeneity -
+#'   if \code{byvar} is not missing.}  \item{pval.Q.w.fixed}{P-value
+#'   of within subgroups heterogeneity statistic Q (based on fixed
+#'   effect model) - if \code{byvar} is not missing.}
+#'   \item{pval.Q.w.random}{P-value of within subgroups heterogeneity
+#'   statistic Q (based on random effects model) - if \code{byvar} is
+#'   not missing.}  \item{Q.b.fixed}{Overall between subgroups
+#'   heterogeneity statistic Q (based on fixed effect model) - if
+#'   \code{byvar} is not missing.}  \item{Q.b.random}{Overall between
+#'   subgroups heterogeneity statistic Q (based on random effects
+#'   model) - if \code{byvar} is not missing.}  \item{df.Q.b}{Degrees
+#'   of freedom for test of overall between subgroups heterogeneity -
+#'   if \code{byvar} is not missing.}  \item{pval.Q.b.fixed}{P-value
+#'   of between subgroups heterogeneity statistic Q (based on fixed
+#'   effect model) - if \code{byvar} is not missing.}
+#'   \item{pval.Q.b.random}{P-value of between subgroups heterogeneity
+#'   statistic Q (based on random effects model) - if \code{byvar} is
+#'   not missing.}  \item{tau.w}{Square-root of between-study variance
+#'   within subgroups - if \code{byvar} is not missing.}
+#'   \item{H.w}{Heterogeneity statistic H within subgroups - if
+#'   \code{byvar} is not missing.}  \item{lower.H.w, upper.H.w}{Lower
+#'   and upper confidence limit for heterogeneity statistic H within
 #'   subgroups - if \code{byvar} is not missing.}
-#' \item{event.c.w}{Number of events in control group in subgroups -
-#'   if \code{byvar} is not missing.}
-#' \item{n.c.w}{Number of observations in control group in subgroups -
-#'   if \code{byvar} is not missing.}
-#' \item{k.w}{Number of studies combined within subgroups - if
-#'   \code{byvar} is not missing.}
-#' \item{k.all.w}{Number of all studies in subgroups - if \code{byvar}
-#'   is not missing.}
-#' \item{Q.w.fixed}{Overall within subgroups heterogeneity statistic Q
-#'   (based on fixed effect model) - if \code{byvar} is not missing.}
-#' \item{Q.w.random}{Overall within subgroups heterogeneity statistic
-#'   Q (based on random effects model) - if \code{byvar} is not
-#'   missing (only calculated if argument \code{tau.common} is TRUE).}
-#' \item{df.Q.w}{Degrees of freedom for test of overall within
-#'   subgroups heterogeneity - if \code{byvar} is not missing.}
-#' \item{pval.Q.w.fixed}{P-value of within subgroups heterogeneity
-#'   statistic Q (based on fixed effect model) - if \code{byvar} is
-#'   not missing.}
-#' \item{pval.Q.w.random}{P-value of within subgroups heterogeneity
-#'   statistic Q (based on random effects model) - if \code{byvar} is
-#'   not missing.}
-#' \item{Q.b.fixed}{Overall between subgroups heterogeneity statistic
-#'   Q (based on fixed effect model) - if \code{byvar} is not
-#'   missing.}
-#' \item{Q.b.random}{Overall between subgroups heterogeneity statistic
-#'   Q (based on random effects model) - if \code{byvar} is not
-#'   missing.}
-#' \item{df.Q.b}{Degrees of freedom for test of overall between
-#'   subgroups heterogeneity - if \code{byvar} is not missing.}
-#' \item{pval.Q.b.fixed}{P-value of between subgroups heterogeneity
-#'   statistic Q (based on fixed effect model) - if \code{byvar} is
-#'   not missing.}
-#' \item{pval.Q.b.random}{P-value of between subgroups heterogeneity
-#'   statistic Q (based on random effects model) - if \code{byvar} is
-#'   not missing.}
-#' \item{tau.w}{Square-root of between-study variance within subgroups
-#'   - if \code{byvar} is not missing.}
-#' \item{H.w}{Heterogeneity statistic H within subgroups - if
-#'   \code{byvar} is not missing.}
-#' \item{lower.H.w, upper.H.w}{Lower and upper confidence limit for
-#'   heterogeneity statistic H within subgroups - if \code{byvar} is
-#'   not missing.}
-#' \item{I2.w}{Heterogeneity statistic I\eqn{^2} within subgroups - if
-#'   \code{byvar} is not missing.}
-#' \item{lower.I2.w, upper.I2.w}{Lower and upper confidence limit for
-#'   heterogeneity statistic I\eqn{^2} within subgroups - if \code{byvar} is
-#'   not missing.}
-#' \item{keepdata}{As defined above.}
-#' \item{data}{Original data (set) used in function call (if
-#'   \code{keepdata = TRUE}).}
-#' \item{subset}{Information on subset of original data used in
-#'   meta-analysis (if \code{keepdata = TRUE}).}
-#' \item{.glmm.fixed}{GLMM object generated by call of
+#'   \item{I2.w}{Heterogeneity statistic I\eqn{^2} within subgroups -
+#'   if \code{byvar} is not missing.}  \item{lower.I2.w,
+#'   upper.I2.w}{Lower and upper confidence limit for heterogeneity
+#'   statistic I\eqn{^2} within subgroups - if \code{byvar} is not
+#'   missing.}  \item{keepdata}{As defined above.}
+#'   \item{data}{Original data (set) used in function call (if
+#'   \code{keepdata = TRUE}).}  \item{subset}{Information on subset of
+#'   original data used in meta-analysis (if \code{keepdata = TRUE}).}
+#'   \item{.glmm.fixed}{GLMM object generated by call of
 #'   \code{\link[metafor]{rma.glmm}} function (fixed effect model).}
-#' \item{.glmm.random}{GLMM object generated by call of
+#'   \item{.glmm.random}{GLMM object generated by call of
 #'   \code{\link[metafor]{rma.glmm}} function (random effects model).}
-#' \item{call}{Function call.}
-#' \item{version}{Version of R package \bold{meta} used to create
-#'   object.}
-#' \item{version.metafor}{Version of R package \bold{metafor} used for
-#'   GLMMs.}
+#'   \item{call}{Function call.}  \item{version}{Version of R package
+#'   \bold{meta} used to create object.}
+#'   \item{version.metafor}{Version of R package \bold{metafor} used
+#'   for GLMMs.}
 #' 
 #' @author Guido Schwarzer \email{sc@@imbi.uni-freiburg.de}
 #' 
@@ -612,8 +624,8 @@
 #' \bold{172}, 137--59
 #' 
 #' IQWiG (2020):
-#' General Methods: Draft of Version 6.0.
-#' \url{https://www.iqwig.de/en/methods/methods-paper.3020.html}
+#' General Methods: Version 6.0.
+#' \url{https://www.iqwig.de/en/about-us/methods/methods-paper/}
 #' 
 #' Knapp G & Hartung J (2003):
 #' Improved tests for a random effects meta-regression with a single
@@ -621,9 +633,8 @@
 #' \emph{Statistics in Medicine},
 #' \bold{22}, 2693--710
 #' 
-#' \emph{Review Manager (RevMan)}
-#' [Computer program]. Version 5.3.
-#' Copenhagen: The Nordic Cochrane Centre, The Cochrane Collaboration, 2014
+#' \emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
+#' The Cochrane Collaboration, 2020
 #' 
 #' Paule RC & Mandel J (1982):
 #' Consensus values and weighting factors.
@@ -842,7 +853,7 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
                       ifelse(!is.na(charmatch(tolower(method), "glmm",
                                               nomatch = NA)),
                              "ML", gs("method.tau")),
-                    method.tau.ci = if (method.tau == "DL") "J" else "QP",
+                    method.tau.ci = gs("method.tau.ci"),
                     tau.preset = NULL, TE.tau = NULL,
                     tau.common = gs("tau.common"),
                     ##
@@ -855,6 +866,13 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
                     ##
                     backtransf = gs("backtransf"),
                     pscale = 1,
+                    ##
+                    text.fixed = gs("text.fixed"),
+                    text.random = gs("text.random"),
+                    text.predict = gs("text.predict"),
+                    text.w.fixed = gs("text.w.fixed"),
+                    text.w.random = gs("text.w.random"),
+                    ##
                     title = gs("title"), complab = gs("complab"),
                     outclab = "",
                     label.e = gs("label.e"), label.c = gs("label.c"),
@@ -896,6 +914,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   chklogical(hakn)
   adhoc.hakn <- setchar(adhoc.hakn, .settings$adhoc4hakn)
   method.tau <- setchar(method.tau, c(.settings$meth4tau, "KD"))
+  if (is.null(method.tau.ci))
+    method.tau.ci <- if (method.tau == "DL") "J" else "QP"
   method.tau.ci <- setchar(method.tau.ci, .settings$meth4tau.ci)
   chklogical(tau.common)
   ##
@@ -908,6 +928,17 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   ##
   chknumeric(pscale, length = 1)
   ##
+  if (!is.null(text.fixed))
+    chkchar(text.fixed, length = 1)
+  if (!is.null(text.random))
+    chkchar(text.random, length = 1)
+  if (!is.null(text.predict))
+    chkchar(text.predict, length = 1)
+  if (!is.null(text.w.fixed))
+    chkchar(text.w.fixed, length = 1)
+  if (!is.null(text.w.random))
+    chkchar(text.w.random, length = 1)
+  ##
   chklogical(keepdata)
   ##
   ## Additional arguments / checks for metabin objects
@@ -1501,6 +1532,9 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
   k <- sum(!is.na(event.e[!exclude]) & !is.na(event.c[!exclude]) &
            !is.na(n.e[!exclude]) & !is.na(n.c[!exclude]))
   ##
+  if (k == 1 & hakn)
+    hakn <- FALSE
+  ##
   if (all(incr.e == 0) & all(incr.c == 0) & method == "MH" & MH.exact == FALSE)
     MH.exact <- TRUE
   ##
@@ -1562,7 +1596,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
       R <- ((n11 + incr.e) * (n12 + incr.e) * (n2. + 2 * incr.c)^3 +
               (n21 + incr.c) * (n22 + incr.c) * (n1. + 2 * incr.e)^3) /
                 ((n1. + 2 * incr.e) * (n2. + 2 * incr.c) * (n.. + 2 * incr.e + 2 * incr.c)^2)
-      S <- n1. * n2. / n..
+      S <- (n1. + 2 * incr.e) * (n2. + 2 * incr.c) /
+        (n.. + 2 * incr.e + 2 * incr.c)
       ##
       R[exclude] <- S[exclude] <- 0
       ##
@@ -1582,13 +1617,24 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     w.fixed[is.na(w.fixed)] <- 0
   }
   else if (is.glmm) {
-    glmm.fixed <- rma.glmm(ai = event.e[!exclude], n1i = n.e[!exclude],
-                           ci = event.c[!exclude], n2i = n.c[!exclude],
-                           method = "FE", test = ifelse(hakn, "t", "z"),
-                           level = 100 * level.comb,
-                           measure = "OR", model = model.glmm,
-                           control = control,
-                           ...)
+    zero.all <-
+      (sum(event.e[!exclude], na.rm = TRUE) == 0 &
+       sum(event.c[!exclude], na.rm = TRUE) == 0) |
+      (!any(event.e[!exclude] != n.e[!exclude]) |
+       !any(event.c[!exclude] != n.c[!exclude]))
+    ##
+    if (!zero.all)
+      glmm.fixed <- rma.glmm(ai = event.e[!exclude], n1i = n.e[!exclude],
+                             ci = event.c[!exclude], n2i = n.c[!exclude],
+                             method = "FE", test = ifelse(hakn, "t", "z"),
+                             level = 100 * level.comb,
+                             measure = "OR", model = model.glmm,
+                             control = control,
+                             ...)
+    else
+      glmm.fixed <- list(b = NA, se = NA,
+                         QE.Wld = NA, QE.df = NA, QE.LRT = NA,
+                         tau2 = NA, se.tau2 = NA)
     ##
     TE.fixed   <- as.numeric(glmm.fixed$b)
     seTE.fixed <- as.numeric(glmm.fixed$se)
@@ -1628,6 +1674,11 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
                method.bias = method.bias,
                ##
                backtransf = backtransf,
+               ##
+               text.fixed = text.fixed, text.random = text.random,
+               text.predict = text.predict,
+               text.w.fixed = text.w.fixed, text.w.random = text.w.random,
+               ##
                title = title, complab = complab, outclab = outclab,
                label.e = label.e, label.c = label.c,
                label.left = label.left, label.right = label.right,
@@ -1647,7 +1698,7 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     w.random[is.na(w.random)] <- 0
   }
   ##
-  if (by & tau.common) {
+  if (by & tau.common & !is.glmm) {
     ## Estimate common tau-squared across subgroups
     hcc <- hetcalc(TE, seTE, method.tau, "",
                    if (Q.Cochrane & method == "MH") TE.fixed else TE.tau,
@@ -1704,22 +1755,24 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     res$upper.fixed <- ci.f$upper
     res$statistic.fixed <- ci.f$statistic
     res$pval.fixed <- ci.f$p
+    res$zval.fixed <- ci.f$statistic
   }
   ##
   if (is.glmm) {
     ##
-    if (sum(!exclude) > 1)
+    if (sum(!exclude) > 1 & !zero.all)
       glmm.random <- rma.glmm(ai = event.e[!exclude], n1i = n.e[!exclude],
                               ci = event.c[!exclude], n2i = n.c[!exclude],
                               method = method.tau,
                               test = ifelse(hakn, "t", "z"),
                               level = 100 * level.comb,
-                              measure = "OR", model = model.glmm,
+                                measure = "OR", model = model.glmm,
                               control = control,
                               ...)
     else {
       ##
       ## Fallback to fixed effect model due to small number of studies
+      ## or zero or all events in all studies
       ##
       glmm.random <- glmm.fixed
     }
@@ -1727,7 +1780,8 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     TE.random   <- as.numeric(glmm.random$b)
     seTE.random <- as.numeric(glmm.random$se)
     ##
-    ci.r <- ci(TE.random, seTE.random, level = level.comb)
+    ci.r <- ci(TE.random, seTE.random, level = level.comb,
+               df = if (hakn) k - 1)
     ##
     res$w.random <- rep(NA, length(event.e))
     ##
@@ -1737,6 +1791,7 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     res$upper.random <- ci.r$upper
     res$statistic.random <- ci.r$statistic
     res$pval.random <- ci.r$p
+    res$zval.random <- ci.r$statistic
     ##
     ## Prediction interval
     ##
@@ -1782,13 +1837,15 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     res$sign.lower.tau <- ""
     res$sign.upper.tau <- ""
     ##
-    res$H <- sqrt(glmm.random$H2)
-    res$lower.H <- NA
-    res$upper.H <- NA
+    H <- calcH(res$Q, res$df.Q, level.comb)
+    res$H <- H$TE
+    res$lower.H <- H$lower
+    res$upper.H <- H$upper
     ##
-    res$I2 <- glmm.random$I2 / 100
-    res$lower.I2 <- NA
-    res$upper.I2 <- NA
+    I2 <- isquared(res$Q, res$df.Q, level.comb)
+    res$I2 <- I2$TE
+    res$lower.I2 <- I2$lower
+    res$upper.I2 <- I2$upper
     ##
     res$Rb <- NA
     res$lower.Rb <- NA
@@ -1797,6 +1854,77 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     res$.glmm.fixed  <- glmm.fixed
     res$.glmm.random <- glmm.random
     res$version.metafor <- packageDescription("metafor")$Version
+    ##
+    if (by) {
+      n.by <- length(unique(byvar[!exclude]))
+      if (n.by > 1)
+        byvar.glmm <- factor(byvar[!exclude], bylevs(byvar[!exclude]))
+      ##
+      glmm.random.by <-
+        try(suppressWarnings(rma.glmm(ai = event.e[!exclude],
+                                      n1i = n.e[!exclude],
+                                      ci = event.c[!exclude],
+                                      n2i = n.c[!exclude],
+                                      mods =
+                                        if (n.by > 1) ~ byvar.glmm else NULL,
+                                      method = method.tau,
+                                      test = ifelse(hakn, "t", "z"),
+                                      level = 100 * level.comb,
+                                      measure = "OR", model = model.glmm,
+                                      control = control,
+                                      ...)),
+            silent = TRUE)
+      ##
+      if ("try-error" %in% class(glmm.random.by))
+        if (grepl(paste0("Number of parameters to be estimated is ",
+                         "larger than the number of observations"),
+                  glmm.random.by)) {
+          glmm.random.by <-
+            suppressWarnings(rma.glmm(ai = event.e[!exclude],
+                                      n1i = n.e[!exclude],
+                                      ci = event.c[!exclude],
+                                      n2i = n.c[!exclude],
+                                      mods =
+                                        if (n.by > 1) ~ byvar.glmm else NULL,
+                                      method = "FE",
+                                      test = ifelse(hakn, "t", "z"),
+                                      level = 100 * level.comb,
+                                      measure = "OR", model = model.glmm,
+                                      control = control,
+                                      ...))
+        }
+        else
+          stop(glmm.random.by)
+      ##
+      Q.r <- glmm.random.by$QE.Wld
+      df.Q.r <- glmm.random.by$k - glmm.random.by$p
+      ##
+      H.r  <- calcH(Q.r, df.Q.r, level.comb)
+      I2.r <- isquared(Q.r, df.Q.r, level.comb)
+      ##
+      hcc <- list(tau2.resid = glmm.random.by$tau2,
+                  lower.tau2.resid = NA,
+                  upper.tau2.resid = NA,
+                  ##
+                  tau.resid = sqrt(glmm.random.by$tau2),
+                  lower.tau.resid = NA,
+                  upper.tau.resid = NA,
+                  sign.lower.tau.resid = "",
+                  sign.upper.tau.resid = "",
+                  ##
+                  Q.resid = NA,
+                  df.Q.resid = NA,
+                  pval.Q.resid = NA,
+                  ##
+                  H.resid = H.r$TE,
+                  lower.H.resid = H.r$lower,
+                  upper.H.resid = H.r$upper,
+                  ##
+                  I2.resid = I2.r$TE,
+                  lower.I2.resid = I2.r$lower,
+                  upper.I2.resid = I2.r$upper
+                  )
+    }
   }
   else if (method == "SSW") {
     if (hakn)
@@ -1811,6 +1939,7 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     res$upper.random <- ci.r$upper
     res$statistic.random <- ci.r$statistic
     res$pval.random <- ci.r$p      
+    res$zval.random <- ci.r$statistic
   }
   ##
   if (keepdata) {
@@ -1830,74 +1959,48 @@ metabin <- function(event.e, n.e, event.c, n.c, studlab,
     res$byseparator <- byseparator
     res$tau.common <- tau.common
     ##
-    if (!tau.common) {
+    if (!tau.common)
       res <- c(res, subgroup(res))
-      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
-      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
-    }
-    else if (!is.null(tau.preset)) {
+    else if (!is.null(tau.preset))
       res <- c(res, subgroup(res, tau.preset))
-      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
-      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
-    }
     else {
-      if (is.glmm) {
+      if (is.glmm)
         res <- c(res, subgroup(res, NULL,
                                factor(res$byvar, bylevs(res$byvar)), ...))
-        res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
-        res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
-      }
-      else {
-        res <- c(res, subgroup(res, hcc$tau))
-        res$Q.w.random <- hcc$Q
-        res$df.Q.w.random <- hcc$df.Q
-        res$tau2.resid <- hcc$tau2
-        res$lower.tau2.resid <- hcc$lower.tau2
-        res$upper.tau2.resid <- hcc$upper.tau2
-        res$tau.resid <- hcc$tau
-        res$lower.tau.resid <- hcc$lower.tau
-        res$upper.tau.resid <- hcc$upper.tau
-        res$sign.lower.tau.resid <- hcc$sign.lower.tau
-        res$sign.upper.tau.resid <- hcc$sign.upper.tau
-      }
+      else
+        res <- c(res, subgroup(res, hcc$tau.resid))
     }
     ##
-    if (!tau.common || method.tau == "DL") {
-      ci.H.resid <- calcH(res$Q.w.fixed, res$df.Q.w, level.comb)
+    if (!tau.common || !is.null(tau.preset)) {
+      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
+      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
       ##
-      res$H.resid <- ci.H.resid$TE
-      res$lower.H.resid <- ci.H.resid$lower
-      res$upper.H.resid <- ci.H.resid$upper
-      ##
-      ci.I2.resid <- isquared(res$Q.w.fixed, res$df.Q.w, level.comb)
-      ##
-      res$I2.resid <- ci.I2.resid$TE
-      res$lower.I2.resid <- ci.I2.resid$lower
-      res$upper.I2.resid <- ci.I2.resid$upper
+      res$Q.resid <- res$df.Q.resid <- res$pval.Q.resid <- NA
+      res$H.resid <- res$lower.H.resid <- res$upper.H.resid <- NA
+      res$I2.resid <- res$lower.I2.resid <- res$upper.I2.resid <- NA
     }
     else {
-      if (is.glmm) {
-        ci.H.resid <- calcH(res$Q.w.fixed, res$df.Q.w, level.comb)
-        ##
-        res$H.resid <- ci.H.resid$TE
-        res$lower.H.resid <- ci.H.resid$lower
-        res$upper.H.resid <- ci.H.resid$upper
-        ##
-        ci.I2.resid <- isquared(res$Q.w.fixed, res$df.Q.w, level.comb)
-        ##
-        res$I2.resid <- ci.I2.resid$TE
-        res$lower.I2.resid <- ci.I2.resid$lower
-        res$upper.I2.resid <- ci.I2.resid$upper
-      }
-      else {
-        res$H.resid <- hcc$H.resid
-        res$lower.H.resid <- hcc$lower.H.resid
-        res$upper.H.resid <- hcc$upper.H.resid
-        ##
-        res$I2.resid <- hcc$I2.resid
-        res$lower.I2.resid <- hcc$lower.I2.resid
-        res$upper.I2.resid <- hcc$upper.I2.resid
-      }
+      res$tau2.resid <- hcc$tau2.resid
+      res$lower.tau2.resid <- hcc$lower.tau2.resid
+      res$upper.tau2.resid <- hcc$upper.tau2.resid
+      ##
+      res$tau.resid <- hcc$tau.resid
+      res$lower.tau.resid <- hcc$lower.tau.resid
+      res$upper.tau.resid <- hcc$upper.tau.resid
+      res$sign.lower.tau.resid <- hcc$sign.lower.tau.resid
+      res$sign.upper.tau.resid <- hcc$sign.upper.tau.resid
+      ##
+      res$Q.w.random <- hcc$Q.resid
+      res$df.Q.w.random <- hcc$df.Q.resid
+      res$pval.Q.w.random <- hcc$pval.Q.resid
+      ##
+      res$H.resid <- hcc$H.resid
+      res$lower.H.resid <- hcc$lower.H.resid
+      res$upper.H.resid <- hcc$upper.H.resid
+      ##
+      res$I2.resid <- hcc$I2.resid
+      res$lower.I2.resid <- hcc$lower.I2.resid
+      res$upper.I2.resid <- hcc$upper.I2.resid
     }
     ##
     res$event.w <- NULL
