@@ -17,6 +17,9 @@
 #' @param exclude An optional vector specifying studies to exclude
 #'   from meta-analysis, however, to include in printouts and forest
 #'   plots (see Details).
+#' @param id An optional vector specifying which estimates come from
+#'   the same study resulting in the use of a three-level
+#'   meta-analysis model.
 #' @param sm A character string indicating underlying summary measure,
 #'   e.g., \code{"RD"}, \code{"RR"}, \code{"OR"}, \code{"ASD"},
 #'   \code{"HR"}, \code{"MD"}, \code{"SMD"}, or \code{"ROM"}.
@@ -82,8 +85,8 @@
 #' @param adhoc.hakn A character string indicating whether an \emph{ad
 #'   hoc} variance correction should be applied in the case of an
 #'   arbitrarily small Hartung-Knapp variance estimate. Either
-#'   \code{""}, \code{"se"}, or \code{"ci"} (see Details), can be
-#'   abbreviated.
+#'   \code{""}, \code{"se"}, \code{"ci"}, or \code{"iqwig6"} (see
+#'   Details), can be abbreviated.
 #' @param method.tau A character string indicating which method is
 #'   used to estimate the between-study variance \eqn{\tau^2} and its
 #'   square root \eqn{\tau}. Either \code{"DL"}, \code{"PM"},
@@ -91,14 +94,15 @@
 #'   \code{"HE"}, or \code{"EB"}, can be abbreviated.
 #' @param method.tau.ci A character string indicating which method is
 #'   used to estimate the confidence interval of \eqn{\tau^2} and
-#'   \eqn{\tau}. Either \code{"QP"}, \code{"BJ"}, or \code{"J"}, or
-#'   \code{""}, can be abbreviated.
+#'   \eqn{\tau}. Either \code{"QP"}, \code{"BJ"}, \code{"J"},
+#'   \code{"PL"}, or \code{""}, can be abbreviated.
 #' @param tau.preset Prespecified value for the square root of the
 #'   between-study variance \eqn{\tau^2}.
 #' @param TE.tau Overall treatment effect used to estimate the
 #'   between-study variance tau-squared.
 #' @param tau.common A logical indicating whether tau-squared should
 #'   be the same across subgroups.
+#' @param detail.tau Detail on between-study variance estimate.
 #' @param method.bias A character string indicating which test is to
 #'   be used.  Either \code{"rank"}, \code{"linreg"}, or \code{"mm"},
 #'   can be abbreviated.  See function \code{\link{metabias}}.
@@ -118,6 +122,16 @@
 #'   \code{"IRS"}, \code{"IRFT"}, or \code{"IRD"}.
 #' @param irunit A character specifying the time unit used to
 #'   calculate rates, e.g. person-years.
+#' @param text.fixed A character string used in printouts and forest
+#'   plot to label the pooled fixed effect estimate.
+#' @param text.random A character string used in printouts and forest
+#'   plot to label the pooled random effects estimate.
+#' @param text.predict A character string used in printouts and forest
+#'   plot to label the prediction interval.
+#' @param text.w.fixed A character string used to label weights of
+#'   fixed effect model.
+#' @param text.w.random A character string used to label weights of
+#'   random effects model.
 #' @param title Title of meta-analysis / systematic review.
 #' @param complab Comparison label.
 #' @param outclab Outcome label.
@@ -140,7 +154,8 @@
 #'   standard errors).
 #' @param control An optional list to control the iterative process to
 #'   estimate the between-study variance \eqn{\tau^2}. This argument
-#'   is passed on to \code{\link[metafor]{rma.uni}}.
+#'   is passed on to \code{\link[metafor]{rma.uni}} or
+#'   \code{\link[metafor]{rma.mv}}.
 #' 
 #' @details
 #' This function provides the \emph{generic inverse variance method}
@@ -292,14 +307,22 @@
 #' variance \eqn{\tau^2}.
 #' \tabular{ll}{
 #' \bold{Argument}\tab \bold{Method} \cr 
-#' \code{method.tau = "DL"}\tab DerSimonian-Laird estimator (DerSimonian and Laird, 1986) \cr
-#' \code{method.tau = "PM"}\tab Paule-Mandel estimator (Paule and Mandel, 1982) \cr
-#' \code{method.tau = "REML"}\tab Restricted maximum-likelihood estimator (Viechtbauer, 2005) \cr
-#' \code{method.tau = "ML"}\tab Maximum-likelihood estimator (Viechtbauer, 2005) \cr
-#' \code{method.tau = "HS"}\tab Hunter-Schmidt estimator (Hunter and Schmidt, 2015) \cr
-#' \code{method.tau = "SJ"}\tab Sidik-Jonkman estimator (Sidik and Jonkman, 2005) \cr
-#' \code{method.tau = "HE"}\tab Hedges estimator (Hedges and Olkin, 1985) \cr
-#' \code{method.tau = "EB"}\tab Empirical Bayes estimator (Morris, 1983)
+#' \code{method.tau = "DL"}
+#'  \tab DerSimonian-Laird estimator (DerSimonian and Laird, 1986) \cr
+#' \code{method.tau = "PM"}
+#'  \tab Paule-Mandel estimator (Paule and Mandel, 1982) \cr
+#' \code{method.tau = "REML"}
+#'  \tab Restricted maximum-likelihood estimator (Viechtbauer, 2005) \cr
+#' \code{method.tau = "ML"}
+#'  \tab Maximum-likelihood estimator (Viechtbauer, 2005) \cr
+#' \code{method.tau = "HS"}
+#'  \tab Hunter-Schmidt estimator (Hunter and Schmidt, 2015) \cr
+#' \code{method.tau = "SJ"}
+#'  \tab Sidik-Jonkman estimator (Sidik and Jonkman, 2005) \cr
+#' \code{method.tau = "HE"}
+#'  \tab Hedges estimator (Hedges and Olkin, 1985) \cr
+#' \code{method.tau = "EB"}
+#'  \tab Empirical Bayes estimator (Morris, 1983)
 #' }
 #'
 #' Historically, the DerSimonian-Laird method was the de facto
@@ -321,12 +344,17 @@
 #' \bold{Argument}\tab \bold{Method} \cr 
 #' \code{method.tau.ci = "J"}\tab Method by Jackson (2013) \cr
 #' \code{method.tau.ci = "BJ"}\tab Method by Biggerstaff and Jackson (2008) \cr
-#' \code{method.tau.ci = "QP"}\tab Q-Profile method (Viechtbauer, 2007)
+#' \code{method.tau.ci = "QP"}\tab Q-Profile method (Viechtbauer, 2007) \cr
+#' \code{method.tau.ci = "PL"}\tab Profile-Likelihood method for
+#'  three-level meta-analysis model \cr
+#' \tab (Van den Noortgate et al., 2013)
 #' }
-#' These methods have been recommended by Veroniki et al. (2016). By
-#' default, the Jackson method is used for the DerSimonian-Laird
-#' estimator of \eqn{\tau^2} and the Q-profile method for all other
-#' estimators of \eqn{\tau^2}. No confidence intervals for
+#' The first three methods have been recommended by Veroniki et
+#' al. (2016). By default, the Jackson method is used for the
+#' DerSimonian-Laird estimator of \eqn{\tau^2} and the Q-profile
+#' method for all other estimators of \eqn{\tau^2}. The
+#' Profile-Likelihood method is the only method available for the
+#' three-level meta-analysis model. No confidence intervals for
 #' \eqn{\tau^2} and \eqn{\tau} are calculated if \code{method.tau.ci =
 #' ""}.
 #' }
@@ -343,18 +371,30 @@
 #' arbitrarily small resulting in a very narrow confidence interval
 #' (Knapp and Hartung, 2003; Wiksten et al., 2016). In such cases, an
 #' \emph{ad hoc} variance correction has been proposed by utilising
-#' the variance estimate from the classic random effects model (Knapp
-#' and Hartung, 2003). Argument \code{adhoc.hakn} can be used to
-#' choose the \emph{ad hoc} method:
+#' the variance estimate from the classic random effects model with
+#' the HK method (Knapp and Hartung, 2003; IQWiQ, 2020). An
+#' alternative approach is to use the wider confidence interval of
+#' classic fixed or random effects meta-analysis and the HK method
+#' (Wiksten et al., 2016; Jackson et al., 2017).
+#'
+#' Argument \code{adhoc.hakn} can be used to choose the \emph{ad hoc}
+#' method:
 #' \tabular{ll}{
-#' \bold{Argument}\tab \bold{\emph{Ad hoc} method} \cr 
+#' \bold{Argument}\tab \bold{\emph{Ad hoc} method} \cr
 #' \code{adhoc.hakn = ""}\tab not used \cr
-#' \code{adhoc.hakn = "se"}\tab used if HK standard error is smaller than
-#'  standard error \cr
-#'  \tab from classic random effects model (Knapp and Hartung, 2003) \cr
-#' \code{adhoc.hakn = "ci"}\tab used if HK confidence interval is
-#'  narrower than CI from \cr
-#'  \tab classic random effects model with DL estimator (IQWiG, 2020)
+#' \code{adhoc.hakn = "se"}\tab use variance correction if HK standard
+#'  error is smaller \cr
+#'  \tab than standard error from classic random effects
+#'  \cr
+#'  \tab meta-analysis (Knapp and Hartung, 2003) \cr
+#' \code{adhoc.hakn = "iqwig6"}\tab use variance correction if HK
+#'  confidence interval \cr
+#'  \tab is narrower than CI from classic random effects model \cr
+#'  \tab with DerSimonian-Laird estimator (IQWiG, 2020) \cr
+#' \code{adhoc.hakn = "ci"}\tab use wider confidence interval of
+#'  classic random effects \cr
+#'  \tab and HK meta-analysis \cr
+#'  \tab (Hybrid method 2 in Jackson et al., 2017)
 #' }
 #' }
 #' 
@@ -457,7 +497,7 @@
 #' \code{print}, \code{summary}, and \code{forest} functions. The
 #' object is a list containing the following components:
 #' \item{TE, seTE, studlab, exclude, n.e, n.c}{As defined above.}
-#' \item{sm, method.ci, level, level.comb,}{As defined above.}
+#' \item{id, sm, method.ci, level, level.comb,}{As defined above.}
 #' \item{comb.fixed, comb.random,}{As defined above.}
 #' \item{overall, overall.hetstat,}{As defined above.}
 #' \item{hakn, adhoc.hakn, method.tau, method.tau.ci,}{As defined above.}
@@ -660,8 +700,8 @@
 #' \bold{14}, 25
 #' 
 #' IQWiG (2020):
-#' General Methods: Draft of Version 6.0.
-#' \url{https://www.iqwig.de/en/methods/methods-paper.3020.html}
+#' General Methods: Version 6.0.
+#' \url{https://www.iqwig.de/en/about-us/methods/methods-paper/}
 #'
 #' Jackson D (2013):
 #' Confidence intervals for the between-study variance in random
@@ -669,6 +709,12 @@
 #' statistics.
 #' \emph{Research Synthesis Methods},
 #' \bold{4}, 220--229
+#'
+#' Jackson D, Law M, Rücker G, Schwarzer G (2017): 
+#' The Hartung-Knapp modification for random-effects meta-analysis: A
+#' useful refinement but are there any residual concerns?
+#' \emph{Statistics in Medicine},
+#' \bold{36}, 3923--34
 #' 
 #' Knapp G & Hartung J (2003):
 #' Improved tests for a random effects meta-regression with a single
@@ -700,8 +746,8 @@
 #' \emph{Journal of Research of the National Bureau of Standards},
 #' \bold{87}, 377--85
 #' 
-#' \emph{Review Manager (RevMan)} [Computer program]. Version 5.3.
-#' Copenhagen: The Nordic Cochrane Centre, The Cochrane Collaboration, 2014
+#' \emph{Review Manager (RevMan)} [Computer program]. Version 5.4.
+#' The Cochrane Collaboration, 2020
 #'
 #' Shi J, Luo D, Weng H, Zeng X-T, Lin L, Chu H, et al. (2020):
 #' Optimally estimating the sample standard deviation from the
@@ -737,6 +783,11 @@
 #' \emph{Journal of Statistical Software},
 #' \bold{36}, 1--48
 #'
+#' Van den Noortgate W, López-López JA, Marín-Martínez F, Sánchez-Meca J (2013):
+#' Three-level meta-analysis of dependent effect sizes.
+#' \emph{Behavior Research Methods},
+#' \bold{45}, 576--94
+#'
 #' Wan X, Wang W, Liu J, Tong T (2014):
 #' Estimating the sample mean and standard deviation from the sample
 #' size, median, range and/or interquartile range.
@@ -840,7 +891,7 @@
 
 metagen <- function(TE, seTE, studlab,
                     ##
-                    data = NULL, subset = NULL, exclude = NULL,
+                    data = NULL, subset = NULL, exclude = NULL, id = NULL,
                     ##
                     sm = "",
                     ##
@@ -853,9 +904,10 @@ metagen <- function(TE, seTE, studlab,
                     ##
                     hakn = gs("hakn"), adhoc.hakn = gs("adhoc.hakn"),
                     method.tau = gs("method.tau"),
-                    method.tau.ci = if (method.tau == "DL") "J" else "QP",
+                    method.tau.ci = gs("method.tau.ci"),
                     tau.preset = NULL, TE.tau = NULL,
                     tau.common = gs("tau.common"),
+                    detail.tau = "",
                     ##
                     prediction = gs("prediction"),
                     level.predict = gs("level.predict"),
@@ -876,6 +928,13 @@ metagen <- function(TE, seTE, studlab,
                     backtransf = gs("backtransf"),
                     pscale = 1,
                     irscale = 1, irunit = "person-years",
+                    ##
+                    text.fixed = gs("text.fixed"),
+                    text.random = gs("text.random"),
+                    text.predict = gs("text.predict"),
+                    text.w.fixed = gs("text.w.fixed"),
+                    text.w.random = gs("text.w.random"),
+                    ##
                     title = gs("title"), complab = gs("complab"),
                     outclab = "",
                     label.e = gs("label.e"), label.c = gs("label.c"),
@@ -913,8 +972,21 @@ metagen <- function(TE, seTE, studlab,
   ##
   chklogical(hakn)
   adhoc.hakn <- setchar(adhoc.hakn, .settings$adhoc4hakn)
+  missing.method.tau <- missing(method.tau)
   method.tau <- setchar(method.tau, .settings$meth4tau)
+  ##
+  missing.id <- missing(id)
+  ##
+  missing.method.tau.ci <- missing(method.tau.ci)
+  if (is.null(method.tau.ci))
+    if (method.tau == "DL")
+      method.tau.ci <- "J"
+    else if (!missing.id)
+      method.tau.ci <- "PL"
+    else
+      method.tau.ci <- "QP"
   method.tau.ci <- setchar(method.tau.ci, .settings$meth4tau.ci)
+  ##
   chklogical(tau.common)
   ##
   chklogical(prediction)
@@ -940,6 +1012,17 @@ metagen <- function(TE, seTE, studlab,
     irscale <- 1
   }
   ##
+  if (!is.null(text.fixed))
+    chkchar(text.fixed, length = 1)
+  if (!is.null(text.random))
+    chkchar(text.random, length = 1)
+  if (!is.null(text.predict))
+    chkchar(text.predict, length = 1)
+  if (!is.null(text.w.fixed))
+    chkchar(text.w.fixed, length = 1)
+  if (!is.null(text.w.random))
+    chkchar(text.w.random, length = 1)
+  ##
   chklogical(keepdata)
   ##
   ## Additional arguments / checks
@@ -960,8 +1043,8 @@ metagen <- function(TE, seTE, studlab,
   ##
   mf <- match.call()
   ##
-  ## Catch 'TE', 'seTE', 'median', 'lower', 'upper', 'n.e', and 'n.c'
-  ## from data:
+  ## Catch 'TE', 'seTE', 'median', 'lower', 'upper', 'n.e', 'n.c', and
+  ## 'id' from data:
   ##
   missing.TE <- missing(TE)
   missing.seTE <- missing(seTE)
@@ -990,6 +1073,12 @@ metagen <- function(TE, seTE, studlab,
   upper <- eval(mf[[match("upper", names(mf))]],
                 data, enclos = sys.frame(sys.parent()))
   ##
+  id <- eval(mf[[match("id", names(mf))]],
+             data, enclos = sys.frame(sys.parent()))
+  ##
+  if (!missing.id & is.null(id))
+    missing.id <- TRUE
+  ##
   k.All <- if (!missing.TE)
              length(TE)
            else if (!missing.median)
@@ -1102,6 +1191,8 @@ metagen <- function(TE, seTE, studlab,
     chklength(n.e, k.All, arg)
   if (!is.null(n.c))
     chklength(n.c, k.All, arg)
+  if (!missing.id)
+    chklength(id, k.All, arg)
   if (by) {
     chklogical(print.byvar)
     chkchar(byseparator)
@@ -1215,6 +1306,9 @@ metagen <- function(TE, seTE, studlab,
     if (!missing.exclude)
       data$.exclude <- exclude
     ##
+    if (!missing.id)
+      data$.id <- id
+    ##
     if (!missing.pval)
       data$.pval <- pval
     if (!missing.df)
@@ -1262,6 +1356,9 @@ metagen <- function(TE, seTE, studlab,
     if (!is.null(n.c))
       n.c <- n.c[subset]
     ##
+    if (!missing.id)
+      id <- id[subset]
+    ##
     if (!missing.pval)
       pval <- pval[subset]
     if (!missing.df)
@@ -1308,6 +1405,39 @@ metagen <- function(TE, seTE, studlab,
   ##
   chknumeric(TE)
   chknumeric(seTE, 0)
+  ##
+  ## No three-level meta-analysis conducted if variable 'id' contains
+  ## different values for each estimate
+  ##
+  multi.level <- FALSE
+  ##
+  sel.ni <- !is.infinite(TE) & !is.infinite(seTE)
+  if (!missing.id && length(unique(id[sel.ni])) != length(id[sel.ni]))
+    multi.level <- TRUE
+  ##
+  if (!multi.level & method.tau.ci == "PL") {
+    if (method.tau == "DL")
+      method.tau.ci <- "J"
+    else
+      method.tau.ci <- "QP"
+  }
+  ##
+  if (multi.level) {
+    if (!(method.tau %in% c("REML", "ML"))) {
+      if (!missing.method.tau)
+        warning("For three-level model, argument 'method.tau' set to \"REML\".",
+                call. = FALSE)
+      method.tau <- "REML"
+    }
+    ##
+    if (by & !tau.common) {
+      if (!missing(tau.common))
+        warning("For three-level model, argument 'tau.common' set to ",
+                "\"TRUE\".",
+                call. = FALSE)
+      tau.common <- TRUE
+    }
+  }
   
   
   ##
@@ -1609,6 +1739,8 @@ metagen <- function(TE, seTE, studlab,
       warning("Zero values in seTE replaced by NAs.")
     seTE[!is.na(seTE) & seTE == 0] <- NA
   }
+  ##
+  tau2.calc <- NA
   
   
   ##
@@ -1618,6 +1750,13 @@ metagen <- function(TE, seTE, studlab,
   ##
   k <- sum(!is.na(seTE[!exclude]))
   ##
+  if (!missing.id) {
+    id.incl <- id[!exclude]
+    k.study <- length(unique(id.incl[!is.na(seTE[!exclude])]))
+  }
+  else
+    k.study <- k
+  ##
   seTE.random.hakn.orig <- NULL
   ##
   if (k == 0) {
@@ -1642,9 +1781,12 @@ metagen <- function(TE, seTE, studlab,
     hc <- list(tau2 = NA, se.tau2 = NA, lower.tau2 = NA, upper.tau2 = NA,
                tau = NA, lower.tau = NA, upper.tau = NA,
                method.tau.ci = "", sign.lower.tau = "", sign.upper.tau = "",
+               ##
                Q = NA, df.Q = NA, pval.Q = NA,
                H = NA, lower.H = NA, upper.H = NA,
                I2 = NA, lower.I2 = NA, upper.I2 = NA,
+               ##
+               Q.resid = NA, df.Q.resid = NA, pval.Q.resid = NA,
                H.resid = NA, lower.H.resid = NA, upper.H.resid = NA,
                I2.resid = NA, lower.I2.resid = NA, upper.I2.resid = NA)
   }
@@ -1652,58 +1794,112 @@ metagen <- function(TE, seTE, studlab,
     ## At least two studies to perform Hartung-Knapp method
     if (k == 1 & hakn)
       hakn <- FALSE
+    ##
     ## Estimate tau-squared
+    ##
     hc <- hetcalc(TE[!exclude], seTE[!exclude],
                   method.tau, method.tau.ci,
-                  TE.tau, level.comb, control = control)
+                  TE.tau, level.comb,
+                  control = control, id = id)
     ##
     if (by & tau.common) {
       ## Estimate common tau-squared across subgroups
       hcc <- hetcalc(TE[!exclude], seTE[!exclude],
                      method.tau, method.tau.ci,
-                     TE.tau, level.comb, byvar, control)
+                     TE.tau, level.comb,
+                     byvar = byvar,
+                     control = control, id = id)
     }
     ##
-    if (is.null(tau.preset))
-      tau2.calc <- if (is.na(hc$tau2)) 0 else hc$tau2
-    else {
-      tau2.calc <- tau.preset^2
+    ## Different calculations for three-level models
+    ##
+    if (!multi.level) {
       ##
-      hc$tau2 <- tau.preset^2
-      hc$se.tau2 <- hc$lower.tau2 <- hc$upper.tau2 <- NA
-      hc$tau <- tau.preset
-      hc$lower.tau <- hc$upper.tau <- NA
-      hc$method.tau.ci <- ""
-      hc$sign.lower.tau <- ""
-      hc$sign.upper.tau <- ""
+      ## Classic meta-analysis
+      ##
+      if (is.null(tau.preset))
+        tau2.calc <- if (is.na(hc$tau2)) 0 else hc$tau2
+      else {
+        tau2.calc <- tau.preset^2
+        ##
+        hc$tau2 <- tau.preset^2
+        hc$se.tau2 <- hc$lower.tau2 <- hc$upper.tau2 <- NA
+        hc$tau <- tau.preset
+        hc$lower.tau <- hc$upper.tau <- NA
+        hc$method.tau.ci <- ""
+        hc$sign.lower.tau <- ""
+        hc$sign.upper.tau <- ""
+      }
+      ##
+      ## Fixed effect estimate (Cooper & Hedges, 1994, p. 265-6)
+      ##
+      w.fixed <- 1 / seTE^2
+      w.fixed[is.na(w.fixed) | is.na(TE) | exclude] <- 0
+      ##
+      TE.fixed   <- weighted.mean(TE, w.fixed, na.rm = TRUE)
+      seTE.fixed <- sqrt(1 / sum(w.fixed, na.rm = TRUE))
+      ##
+      ci.f <- ci(TE.fixed, seTE.fixed, level = level.comb,
+                 null.effect = null.effect)
+      statistic.fixed <- ci.f$statistic
+      pval.fixed <- ci.f$p
+      lower.fixed <- ci.f$lower
+      upper.fixed <- ci.f$upper
+      ##
+      ## Random effects estimate (Cooper & Hedges, 1994, p. 265, 274-5
+      ##
+      w.random <- 1 / (seTE^2 + sum(tau2.calc, na.rm = TRUE))
+      w.random[is.na(w.random) | is.na(TE) | exclude] <- 0
+      ##
+      TE.random   <- weighted.mean(TE, w.random, na.rm = TRUE)
+      seTE.random <- sqrt(1 / sum(w.random, na.rm = TRUE))
+    }
+    else {
+      ##
+      ## Three-level meta-analysis
+      ##
+      id.TE <- seq_along(TE)
+      ##
+      ## No fixed effect method for three-level model
+      ##
+      if (comb.fixed) {
+        if (!missing(comb.fixed))
+          warning("Fixed effect model not calculated for three-level model.",
+                  call. = FALSE)
+        comb.fixed <- FALSE
+      }
+      ##
+      w.fixed <- rep_len(NA, length(seTE))
+      ##
+      TE.fixed <- seTE.fixed <- lower.fixed <- upper.fixed <-
+        statistic.fixed <- pval.fixed <- NA
+      ##
+      ## Conduct three-level meta-analysis
+      ##
+      if (by & !tau.common) {
+        if (!missing(tau.common))
+          warning("Argument 'tau.common' set to TRUE for three-level model.",
+                  call. = FALSE)
+        tau.common <- TRUE
+      }
+      ##
+      m4 <- rma.mv(TE, seTE^2,
+                   method = method.tau,
+                   random = ~ 1 | id / id.TE,
+                   control = control)
+      ##
+      w.random <- rep_len(NA, length(w.fixed))
+      ##
+      TE.random <- m4$b
+      seTE.random <- m4$se
+      ##
+      tau2.calc <- sum(m4$tau2)
     }
-    ##
-    ## Fixed effect estimate (Cooper & Hedges, 1994, p. 265-6)
-    ##
-    w.fixed <- 1 / seTE^2
-    w.fixed[is.na(w.fixed) | is.na(TE) | exclude] <- 0
-    ##
-    TE.fixed   <- weighted.mean(TE, w.fixed, na.rm = TRUE)
-    seTE.fixed <- sqrt(1 / sum(w.fixed, na.rm = TRUE))
-    ##
-    ci.f <- ci(TE.fixed, seTE.fixed, level = level.comb,
-               null.effect = null.effect)
-    statistic.fixed <- ci.f$statistic
-    pval.fixed <- ci.f$p
-    lower.fixed <- ci.f$lower
-    upper.fixed <- ci.f$upper
-    ##
-    ## Random effects estimate (Cooper & Hedges, 1994, p. 265, 274-5
-    ##
-    w.random <- 1 / (seTE^2 + tau2.calc)
-    w.random[is.na(w.random) | is.na(TE) | exclude] <- 0
-    ##
-    TE.random   <- weighted.mean(TE, w.random, na.rm = TRUE)
-    seTE.random <- sqrt(1 / sum(w.random, na.rm = TRUE))
     ##
     ## Hartung-Knapp adjustment
     ##
     if (hakn) {
+      alpha <- (1 - level.comb) / 2
       df.hakn <- k - 1
       q <- 1 / (k - 1) * sum(w.random * (TE - TE.random)^2, na.rm = TRUE)
       ##
@@ -1713,11 +1909,25 @@ metagen <- function(TE, seTE, studlab,
         ##
         ## Variance correction if SE_HK < SE_notHK (Knapp and Hartung, 2003)
         ##
-        if (q < 1)
+        if (q < 1) {
           seTE.random.hakn.orig <- seTE.random
-        seTE.random <- sqrt(max(q, 1) /  sum(w.random))
+          seTE.random <- sqrt(1 /  sum(w.random))
+        }
       }
       else if (adhoc.hakn == "ci") {
+        ##
+        ## Use wider confidence interval, i.e., confidence interval
+        ## from classic random effects meta-analysis if HK CI is
+        ## smaller
+        ## (Wiksten et al., 2016; Jackson et al., 2017, hybrid 2)
+        ##
+        if (q > qnorm(alpha) / qt(alpha, df = df.hakn)) {
+          seTE.random.hakn.orig <- seTE.random
+          seTE.random <- sqrt(1 /  sum(w.random))
+          df.hakn <- NULL
+        }
+      }
+      else if (adhoc.hakn == "iqwig6") {
         ##
         ## Variance correction if CI_HK < CI_DL (IQWiG, 2020)
         ##
@@ -1769,10 +1979,13 @@ metagen <- function(TE, seTE, studlab,
     p.upper <- NA
   }
   ##
-  ## Calculate Rb
+  ## Calculate Rb (but not for three-level model)
   ##
-  Rbres <- Rb(seTE[!is.na(seTE)], seTE.random,
-              tau2.calc, hc$Q, hc$df.Q, level.comb)
+  if (length(tau2.calc) == 1)
+    Rbres <- Rb(seTE[!is.na(seTE)], seTE.random,
+                tau2.calc, hc$Q, hc$df.Q, level.comb)
+  else
+    Rbres <- list(TE = NA, lower = NA, upper = NA)
   
   
   ##
@@ -1780,6 +1993,9 @@ metagen <- function(TE, seTE, studlab,
   ## (11) Generate R object
   ##
   ##
+  if (missing(detail.tau) && k != k.study)
+    detail.tau <- c("level 2", "level 1")
+  ##
   res <- list(studlab = studlab,
               ##
               TE = TE, seTE = seTE,
@@ -1789,6 +2005,7 @@ metagen <- function(TE, seTE, studlab,
               pval = ci.study$p,
               df = if (method.ci == "t") df else rep_len(NA, length(TE)),
               w.fixed = w.fixed, w.random = w.random,
+              id = id,
               ##
               TE.fixed = TE.fixed, seTE.fixed = seTE.fixed,
               lower.fixed = lower.fixed, upper.fixed = upper.fixed,
@@ -1806,7 +2023,8 @@ metagen <- function(TE, seTE, studlab,
               lower.predict = p.lower, upper.predict = p.upper,
               level.predict = level.predict,
               ##
-              k = k, Q = hc$Q, df.Q = hc$df.Q, pval.Q = hc$pval.Q,
+              k = k, k.study = k.study,
+              Q = hc$Q, df.Q = hc$df.Q, pval.Q = hc$pval.Q,
               tau2 = hc$tau2, se.tau2 = hc$se.tau2,
               lower.tau2 = hc$lower.tau2, upper.tau2 = hc$upper.tau2,
               tau = hc$tau, lower.tau = hc$lower.tau, upper.tau = hc$upper.tau,
@@ -1834,14 +2052,21 @@ metagen <- function(TE, seTE, studlab,
               hakn = hakn, adhoc.hakn = adhoc.hakn,
               df.hakn = if (hakn) df.hakn else NA,
               seTE.random.hakn.orig = seTE.random.hakn.orig,
-              method.tau = method.tau, method.tau.ci = method.tau.ci,
+              method.tau = method.tau, method.tau.ci = hc$method.tau.ci,
               tau.preset = tau.preset,
-              TE.tau = if (!missing(TE.tau) & method.tau == "DL") TE.tau else NULL,
+              TE.tau =
+                if (!missing(TE.tau) & method.tau == "DL") TE.tau else NULL,
               tau.common = tau.common,
+              detail.tau = detail.tau,
               prediction = prediction,
               method.bias = method.bias,
               n.e = n.e,
               n.c = n.c,
+              ##
+              text.fixed = text.fixed, text.random = text.random,
+              text.predict = text.predict,
+              text.w.fixed = text.w.fixed, text.w.random = text.w.random,
+              ##
               title = title, complab = complab, outclab = outclab,
               label.e = label.e,
               label.c = label.c,
@@ -1868,51 +2093,43 @@ metagen <- function(TE, seTE, studlab,
     res$byvar <- byvar
     res$bylab <- bylab
     ##
-    if (!tau.common) {
+    if (!tau.common)
       res <- c(res, subgroup(res))
-      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
-      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
-    }
-    else if (!is.null(tau.preset)) {
+    else if (!is.null(tau.preset))
       res <- c(res, subgroup(res, tau.preset))
+    else
+      res <- c(res, subgroup(res, hcc$tau.resid))
+    ##
+    if (!tau.common || !is.null(tau.preset)) {
       res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
       res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
+      ##
+      res$Q.resid <- res$df.Q.resid <- res$pval.Q.resid <- NA
+      res$H.resid <- res$lower.H.resid <- res$upper.H.resid <- NA
+      res$I2.resid <- res$lower.I2.resid <- res$upper.I2.resid <- NA
     }
     else {
-      res <- c(res, subgroup(res, hcc$tau))
-      res$Q.w.random <- hcc$Q
-      res$df.Q.w.random <- hcc$df.Q
-      res$tau2.resid <- hcc$tau2
-      res$lower.tau2.resid <- hcc$lower.tau2
-      res$upper.tau2.resid <- hcc$upper.tau2
-      res$tau.resid <- hcc$tau
-      res$lower.tau.resid <- hcc$lower.tau
-      res$upper.tau.resid <- hcc$upper.tau
+      res$Q.w.random <- hcc$Q.resid
+      res$df.Q.w.random <- hcc$df.Q.resid
+      ##
+      res$tau2.resid <- hcc$tau2.resid
+      res$lower.tau2.resid <- hcc$lower.tau2.resid
+      res$upper.tau2.resid <- hcc$upper.tau2.resid
+      ##
+      res$tau.resid <- hcc$tau.resid
+      res$lower.tau.resid <- hcc$lower.tau.resid
+      res$upper.tau.resid <- hcc$upper.tau.resid
       res$sign.lower.tau.resid <- hcc$sign.lower.tau
       res$sign.upper.tau.resid <- hcc$sign.upper.tau
-    }
-    ##
-    if (!tau.common || method.tau == "DL") {
-      ci.H.resid <- calcH(res$Q.w.fixed, res$df.Q.w, level.comb)
       ##
-      res$H.resid <- ci.H.resid$TE
-      res$lower.H.resid <- ci.H.resid$lower
-      res$upper.H.resid <- ci.H.resid$upper
-    }
-    else {
+      res$Q.resid <- hcc$Q.resid
+      res$df.Q.resid <- hcc$df.Q.resid
+      res$pval.Q.resid <- hcc$pval.Q.resid
+      ##
       res$H.resid <- hcc$H.resid
       res$lower.H.resid <- hcc$lower.H.resid
       res$upper.H.resid <- hcc$upper.H.resid
-    }
-    ##
-    if (!tau.common || method.tau == "DL") {
-      ci.I2.resid <- isquared(res$Q.w.fixed, res$df.Q.w, level.comb)
       ##
-      res$I2.resid <- ci.I2.resid$TE
-      res$lower.I2.resid <- ci.I2.resid$lower
-      res$upper.I2.resid <- ci.I2.resid$upper
-    }
-    else {
       res$I2.resid <- hcc$I2.resid
       res$lower.I2.resid <- hcc$lower.I2.resid
       res$upper.I2.resid <- hcc$upper.I2.resid
