@@ -86,6 +86,16 @@
 #'   rates.
 #' @param irunit A character string specifying the time unit used to
 #'   calculate rates, e.g. person-years.
+#' @param text.fixed A character string used in printouts and forest
+#'   plot to label the pooled fixed effect estimate.
+#' @param text.random A character string used in printouts and forest
+#'   plot to label the pooled random effects estimate.
+#' @param text.predict A character string used in printouts and forest
+#'   plot to label the prediction interval.
+#' @param text.w.fixed A character string used to label weights of
+#'   fixed effect model.
+#' @param text.w.random A character string used to label weights of
+#'   random effects model.
 #' @param title Title of meta-analysis / systematic review.
 #' @param complab Comparison label.
 #' @param outclab Outcome label.
@@ -208,20 +218,32 @@
 #' In rare settings with very homogeneous treatment estimates, the
 #' Hartung-Knapp variance estimate can be arbitrarily small resulting
 #' in a very narrow confidence interval (Knapp and Hartung, 2003;
-#' Wiksten et al., 2016). In such cases, an \emph{ad hoc} variance
-#' correction has been proposed by utilising the variance estimate
-#' from the classic random effects model (Knapp and Hartung,
-#' 2003). Argument \code{adhoc.hakn} can be used to choose the
-#' \emph{ad hoc} method:
+#' Wiksten et al., 2016). In such cases, an
+#' \emph{ad hoc} variance correction has been proposed by utilising
+#' the variance estimate from the classic random effects model with
+#' the HK method (Knapp and Hartung, 2003; IQWiQ, 2020). An
+#' alternative approach is to use the wider confidence interval of
+#' classic fixed or random effects meta-analysis and the HK method
+#' (Wiksten et al., 2016; Jackson et al., 2017).
+#'
+#' Argument \code{adhoc.hakn} can be used to choose the \emph{ad hoc}
+#' method:
 #' \tabular{ll}{
-#' \bold{Argument}\tab \bold{\emph{Ad hoc} method} \cr 
+#' \bold{Argument}\tab \bold{\emph{Ad hoc} method} \cr
 #' \code{adhoc.hakn = ""}\tab not used \cr
-#' \code{adhoc.hakn = "se"}\tab used if HK standard error is smaller than
-#'  standard error \cr
-#'  \tab from classic random effects model (Knapp and Hartung, 2003) \cr
-#' \code{adhoc.hakn = "ci"}\tab used if HK confidence interval is
-#'  narrower than CI from \cr
-#'  \tab classic random effects model with DL estimator (IQWiG, 2020)
+#' \code{adhoc.hakn = "se"}\tab use variance correction if HK standard
+#'  error is smaller \cr
+#'  \tab than standard error from classic random effects
+#'  \cr
+#'  \tab meta-analysis (Knapp and Hartung, 2003) \cr
+#' \code{adhoc.hakn = "iqwig6"}\tab use variance correction if HK
+#'  confidence interval \cr
+#'  \tab is narrower than CI from classic random effects model \cr
+#'  \tab with DerSimonian-Laird estimator (IQWiG, 2020) \cr
+#' \code{adhoc.hakn = "ci"}\tab use wider confidence interval of
+#'  classic random effects \cr
+#'  \tab and HK meta-analysis \cr
+#'  \tab (Hybrid method 2 in Jackson et al., 2017)
 #' }
 #' }
 #' 
@@ -318,13 +340,13 @@
 #'   incidence rate and standard error (fixed effect model).}
 #' \item{lower.fixed, upper.fixed}{Lower and upper confidence interval
 #'   limits (fixed effect model).}
-#' \item{zval.fixed, pval.fixed}{z-value and p-value for test of
+#' \item{statistic.fixed, pval.fixed}{z-value and p-value for test of
 #'   overall effect (fixed effect model).}
 #' \item{TE.random, seTE.random}{Estimated overall (un)transformed
 #'   incidence rate and standard error (random effects model).}
 #' \item{lower.random, upper.random}{Lower and upper confidence
 #'   interval limits (random effects model).}
-#' \item{zval.random, pval.random}{z-value or t-value and
+#' \item{statistic.random, pval.random}{z-value or t-value and
 #'   corresponding p-value for test of overall effect (random effects
 #'   model).}
 #' \item{prediction, level.predict}{As defined above.}
@@ -368,8 +390,8 @@
 #' \item{lower.fixed.w, upper.fixed.w}{Lower and upper confidence
 #'   interval limits in subgroups (fixed effect model) - if
 #'   \code{byvar} is not missing.}
-#' \item{zval.fixed.w, pval.fixed.w}{z-value and p-value for test of
-#'   treatment effect in subgroups (fixed effect model) - if
+#' \item{statistic.fixed.w, pval.fixed.w}{z-value and p-value for test
+#'   of treatment effect in subgroups (fixed effect model) - if
 #'   \code{byvar} is not missing.}
 #' \item{TE.random.w, seTE.random.w}{Estimated treatment effect and
 #'   standard error in subgroups (random effects model) - if
@@ -377,7 +399,7 @@
 #' \item{lower.random.w, upper.random.w}{Lower and upper confidence
 #'   interval limits in subgroups (random effects model) - if
 #'   \code{byvar} is not missing.}
-#' \item{zval.random.w, pval.random.w}{z-value or t-value and
+#' \item{statistic.random.w, pval.random.w}{z-value or t-value and
 #'   corresponding p-value for test of treatment effect in subgroups
 #'   (random effects model) - if \code{byvar} is not missing.}
 #' \item{w.fixed.w, w.random.w}{Weight of subgroups (in fixed and
@@ -493,9 +515,15 @@
 #' \bold{14}, 25
 #' 
 #' IQWiG (2020):
-#' General Methods: Draft of Version 6.0.
-#' \url{https://www.iqwig.de/en/methods/methods-paper.3020.html}
+#' General Methods: Version 6.0.
+#' \url{https://www.iqwig.de/en/about-us/methods/methods-paper/}
 #'
+#' Jackson D, Law M, Rücker G, Schwarzer G (2017): 
+#' The Hartung-Knapp modification for random-effects meta-analysis: A
+#' useful refinement but are there any residual concerns?
+#' \emph{Statistics in Medicine},
+#' \bold{36}, 3923--34
+#' 
 #' Langan D, Higgins JPT, Jackson D, Bowden J, Veroniki AA,
 #' Kontopantelis E, et al. (2019):
 #' A comparison of heterogeneity variance estimators in simulated
@@ -568,7 +596,7 @@ metarate <- function(event, time, studlab,
                      ##
                      hakn = gs("hakn"), adhoc.hakn = gs("adhoc.hakn"),
                      method.tau,
-                     method.tau.ci = if (method.tau == "DL") "J" else "QP",
+                     method.tau.ci = gs("method.tau.ci"),
                      tau.preset = NULL, TE.tau = NULL,
                      tau.common = gs("tau.common"),
                      ##
@@ -581,6 +609,13 @@ metarate <- function(event, time, studlab,
                      ##
                      backtransf = gs("backtransf"),
                      irscale = 1, irunit = "person-years",
+                     ##
+                     text.fixed = gs("text.fixed"),
+                     text.random = gs("text.random"),
+                     text.predict = gs("text.predict"),
+                     text.w.fixed = gs("text.w.fixed"),
+                     text.w.random = gs("text.w.random"),
+                     ##
                      title = gs("title"), complab = gs("complab"),
                      outclab = "",
                      ##
@@ -615,6 +650,8 @@ metarate <- function(event, time, studlab,
   if (missing(method.tau))
     method.tau <- if (method == "GLMM") "ML" else gs("method.tau")
   method.tau <- setchar(method.tau, .settings$meth4tau)
+  if (is.null(method.tau.ci))
+    method.tau.ci <- if (method.tau == "DL") "J" else "QP"
   method.tau.ci <- setchar(method.tau.ci, .settings$meth4tau.ci)
   chklogical(tau.common)
   ##
@@ -634,13 +671,24 @@ metarate <- function(event, time, studlab,
   }
   chkchar(irunit)
   ##
+  if (!is.null(text.fixed))
+    chkchar(text.fixed, length = 1)
+  if (!is.null(text.random))
+    chkchar(text.random, length = 1)
+  if (!is.null(text.predict))
+    chkchar(text.predict, length = 1)
+  if (!is.null(text.w.fixed))
+    chkchar(text.w.fixed, length = 1)
+  if (!is.null(text.w.random))
+    chkchar(text.w.random, length = 1)
+  ##
   chklogical(keepdata)
   ##
   ## Additional arguments / checks for metainc objects
   ##
   fun <- "metarate"
   ##
-  method <- setchar(method, c("Inverse", "GLMM"))
+  method <- setchar(method, .settings$meth4rate)
   is.glmm <- method == "GLMM"
   ##
   chklogical(allincr)
@@ -923,6 +971,9 @@ metarate <- function(event, time, studlab,
   ##
   k <- sum(!is.na(event[!exclude]) & !is.na(time[!exclude]))
   ##
+  if (k == 1 & hakn)
+    hakn <- FALSE
+  ##
   if (is.glmm & k > 0) {
     glmm.fixed <- rma.glmm(xi = event[!exclude], ti = time[!exclude],
                            method = "FE", test = ifelse(hakn, "t", "z"),
@@ -961,6 +1012,11 @@ metarate <- function(event, time, studlab,
                method.bias = method.bias,
                ##
                backtransf = backtransf,
+               ##
+               text.fixed = text.fixed, text.random = text.random,
+               text.predict = text.predict,
+               text.w.fixed = text.w.fixed, text.w.random = text.w.random,
+               ##
                title = title, complab = complab, outclab = outclab,
                ##
                keepdata = FALSE,
@@ -968,7 +1024,7 @@ metarate <- function(event, time, studlab,
                ##
                control = control)
   ##
-  if (by & tau.common) {
+  if (by & tau.common & !is.glmm) {
     ## Estimate common tau-squared across subgroups
     hcc <- hetcalc(TE, seTE, method.tau, "",
                    TE.tau, level.comb, byvar, control)
@@ -1015,8 +1071,11 @@ metarate <- function(event, time, studlab,
     res$upper.fixed <- ci.f$upper
     res$statistic.fixed <- ci.f$statistic
     res$pval.fixed <- ci.f$statistic
+    res$zval.fixed <- ci.f$statistic
     ##
-    if (sum(!exclude) > 1)
+    if (sum(!exclude) > 1 &
+        sum(event[!exclude], na.rm = TRUE) > 0 &
+        any(event[!exclude] != time[!exclude]))
       glmm.random <- rma.glmm(xi = event[!exclude], ti = time[!exclude],
                               method = method.tau,
                               test = ifelse(hakn, "t", "z"),
@@ -1026,6 +1085,7 @@ metarate <- function(event, time, studlab,
     else {
       ##
       ## Fallback to fixed effect model due to small number of studies
+      ## or zero events (or number of events equal to time at risk)
       ##
       glmm.random <- glmm.fixed
     }
@@ -1034,7 +1094,7 @@ metarate <- function(event, time, studlab,
     seTE.random <- as.numeric(glmm.random$se)
     ##
     ci.r <- ci(TE.random, seTE.random, level = level.comb,
-               null.effect = transf.null.effect)
+               null.effect = transf.null.effect, df = if (hakn) k - 1)
     ##
     res$w.random <- rep(NA, length(event))
     ##
@@ -1044,6 +1104,7 @@ metarate <- function(event, time, studlab,
     res$upper.random <- ci.r$upper
     res$statistic.random <- ci.r$statistic
     res$pval.random <- ci.r$p
+    res$zval.random <- ci.r$statistic
     ##
     ## Prediction interval
     ##
@@ -1087,13 +1148,15 @@ metarate <- function(event, time, studlab,
     res$sign.lower.tau <- ""
     res$sign.upper.tau <- ""
     ##
-    res$H <- sqrt(glmm.random$H2)
-    res$lower.H <- NA
-    res$upper.H <- NA
+    H <- calcH(res$Q, res$df.Q, level.comb)
+    res$H <- H$TE
+    res$lower.H <- H$lower
+    res$upper.H <- H$upper
     ##
-    res$I2 <- glmm.random$I2 / 100
-    res$lower.I2 <- NA
-    res$upper.I2 <- NA
+    I2 <- isquared(res$Q, res$df.Q, level.comb)
+    res$I2 <- I2$TE
+    res$lower.I2 <- I2$lower
+    res$upper.I2 <- I2$upper
     ##
     res$Rb <- NA
     res$lower.Rb <- NA
@@ -1102,6 +1165,71 @@ metarate <- function(event, time, studlab,
     res$.glmm.fixed  <- glmm.fixed
     res$.glmm.random <- glmm.random
     res$version.metafor <- packageDescription("metafor")$Version
+    ##
+    if (by) {
+      n.by <- length(unique(byvar[!exclude]))
+      if (n.by > 1)
+        byvar.glmm <- factor(byvar[!exclude], bylevs(byvar[!exclude]))
+      ##
+      glmm.random.by <-
+        try(suppressWarnings(rma.glmm(xi = event[!exclude],
+                                      ti = time[!exclude],
+                                      mods =
+                                        if (n.by > 1) ~ byvar.glmm else NULL,
+                                      method = method.tau,
+                                      test = ifelse(hakn, "t", "z"),
+                                      level = 100 * level.comb,
+                                      measure = "IRLN", control = control,
+                                      ...)),
+            silent = TRUE)
+      ##
+      if ("try-error" %in% class(glmm.random.by))
+        if (grepl(paste0("Number of parameters to be estimated is ",
+                         "larger than the number of observations"),
+                  glmm.random.by)) {
+          glmm.random.by <-
+            suppressWarnings(rma.glmm(xi = event[!exclude],
+                                      ti = time[!exclude],
+                                      mods =
+                                        if (n.by > 1) ~ byvar.glmm else NULL,
+                                      method = "FE",
+                                      test = ifelse(hakn, "t", "z"),
+                                      level = 100 * level.comb,
+                                      measure = "IRLN", control = control,
+                                      ...))
+        }
+        else
+          stop(glmm.random.by)
+      ##
+      Q.r <- glmm.random.by$QE.Wld
+      df.Q.r <- glmm.random.by$k - glmm.random.by$p
+      ##
+      H.r  <- calcH(Q.r, df.Q.r, level.comb)
+      I2.r <- isquared(Q.r, df.Q.r, level.comb)
+      ##
+      hcc <- list(tau2.resid = glmm.random.by$tau2,
+                  lower.tau2.resid = NA,
+                  upper.tau2.resid = NA,
+                  ##
+                  tau.resid = sqrt(glmm.random.by$tau2),
+                  lower.tau.resid = NA,
+                  upper.tau.resid = NA,
+                  sign.lower.tau.resid = "",
+                  sign.upper.tau.resid = "",
+                  ##
+                  Q.resid = NA,
+                  df.Q.resid = NA,
+                  pval.Q.resid = NA,
+                  ##
+                  H.resid = H.r$TE,
+                  lower.H.resid = H.r$lower,
+                  upper.H.resid = H.r$upper,
+                  ##
+                  I2.resid = I2.r$TE,
+                  lower.I2.resid = I2.r$lower,
+                  upper.I2.resid = I2.r$upper
+                  )
+    }
   }
   ##
   res$lower <- lower.study
@@ -1129,74 +1257,48 @@ metarate <- function(event, time, studlab,
     res$byseparator <- byseparator
     res$tau.common <- tau.common
     ##
-    if (!tau.common) {
+    if (!tau.common)
       res <- c(res, subgroup(res))
-      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
-      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
-    }
-    else if (!is.null(tau.preset)) {
+    else if (!is.null(tau.preset))
       res <- c(res, subgroup(res, tau.preset))
-      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
-      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
-    }
     else {
-      if (is.glmm) {
+      if (is.glmm)
         res <- c(res, subgroup(res, NULL,
                                factor(res$byvar, bylevs(res$byvar)), ...))
-        res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
-        res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
-      }
-      else {
-        res <- c(res, subgroup(res, hcc$tau))
-        res$Q.w.random <- hcc$Q
-        res$df.Q.w.random <- hcc$df.Q
-        res$tau2.resid <- hcc$tau2
-        res$lower.tau2.resid <- hcc$lower.tau2
-        res$upper.tau2.resid <- hcc$upper.tau2
-        res$tau.resid <- hcc$tau
-        res$lower.tau.resid <- hcc$lower.tau
-        res$upper.tau.resid <- hcc$upper.tau
-        res$sign.lower.tau.resid <- hcc$sign.lower.tau
-        res$sign.upper.tau.resid <- hcc$sign.upper.tau
-      }
+      else
+        res <- c(res, subgroup(res, hcc$tau.resid))
     }
     ##
-    if (!tau.common || method.tau == "DL") {
-      ci.H.resid <- calcH(res$Q.w.fixed, res$df.Q.w, level.comb)
+    if (!tau.common || !is.null(tau.preset)) {
+      res$tau2.resid <- res$lower.tau2.resid <- res$upper.tau2.resid <- NA
+      res$tau.resid <- res$lower.tau.resid <- res$upper.tau.resid <- NA
       ##
-      res$H.resid <- ci.H.resid$TE
-      res$lower.H.resid <- ci.H.resid$lower
-      res$upper.H.resid <- ci.H.resid$upper
-      ##
-      ci.I2.resid <- isquared(res$Q.w.fixed, res$df.Q.w, level.comb)
-      ##
-      res$I2.resid <- ci.I2.resid$TE
-      res$lower.I2.resid <- ci.I2.resid$lower
-      res$upper.I2.resid <- ci.I2.resid$upper
+      res$Q.resid <- res$df.Q.resid <- res$pval.Q.resid <- NA
+      res$H.resid <- res$lower.H.resid <- res$upper.H.resid <- NA
+      res$I2.resid <- res$lower.I2.resid <- res$upper.I2.resid <- NA
     }
     else {
-      if (is.glmm) {
-        ci.H.resid <- calcH(res$Q.w.fixed, res$df.Q.w, level.comb)
-        ##
-        res$H.resid <- ci.H.resid$TE
-        res$lower.H.resid <- ci.H.resid$lower
-        res$upper.H.resid <- ci.H.resid$upper
-        ##
-        ci.I2.resid <- isquared(res$Q.w.fixed, res$df.Q.w, level.comb)
-        ##
-        res$I2.resid <- ci.I2.resid$TE
-        res$lower.I2.resid <- ci.I2.resid$lower
-        res$upper.I2.resid <- ci.I2.resid$upper
-      }
-      else {
-        res$H.resid <- hcc$H.resid
-        res$lower.H.resid <- hcc$lower.H.resid
-        res$upper.H.resid <- hcc$upper.H.resid
-        ##
-        res$I2.resid <- hcc$I2.resid
-        res$lower.I2.resid <- hcc$lower.I2.resid
-        res$upper.I2.resid <- hcc$upper.I2.resid
-      }
+      res$tau2.resid <- hcc$tau2.resid
+      res$lower.tau2.resid <- hcc$lower.tau2.resid
+      res$upper.tau2.resid <- hcc$upper.tau2.resid
+      ##
+      res$tau.resid <- hcc$tau.resid
+      res$lower.tau.resid <- hcc$lower.tau.resid
+      res$upper.tau.resid <- hcc$upper.tau.resid
+      res$sign.lower.tau.resid <- hcc$sign.lower.tau.resid
+      res$sign.upper.tau.resid <- hcc$sign.upper.tau.resid
+      ##
+      res$Q.w.random <- hcc$Q.resid
+      res$df.Q.w.random <- hcc$df.Q.resid
+      res$pval.Q.w.random <- hcc$pval.Q.resid
+      ##
+      res$H.resid <- hcc$H.resid
+      res$lower.H.resid <- hcc$lower.H.resid
+      res$upper.H.resid <- hcc$upper.H.resid
+      ##
+      res$I2.resid <- hcc$I2.resid
+      res$lower.I2.resid <- hcc$lower.I2.resid
+      res$upper.I2.resid <- hcc$upper.I2.resid
     }
     ##
     res$event.e.w <- NULL
